@@ -12,6 +12,7 @@
 
 #include "ThreadLabels.h"
 #include "RtsUtils.h"
+#include "RtsFlags.h"
 #include "Hash.h"
 #include "Trace.h"
 
@@ -64,6 +65,6 @@ labelThread(Capability  *cap,
         }
     }
     recordClosureMutated(cap, (StgClosure*)tso);
-    tso->label = label;
+    RELEASE_STORE(&tso->label, label);
     traceThreadLabel(cap, tso, (char *) label->payload, label->bytes);
 }
